'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _program = require('./program');

var _program2 = _interopRequireDefault(_program);

var _utils = require('../utils');

var _edgeFastVert = require('../shaders/edge.fast.vert.glsl');

var _edgeFastVert2 = _interopRequireDefault(_edgeFastVert);

var _edgeFastFrag = require('../shaders/edge.fast.frag.glsl');

var _edgeFastFrag2 = _interopRequireDefault(_edgeFastFrag);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; } /**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * Sigma.js WebGL Renderer Fast Edge Program
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * ==========================================
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * Program rendering edges using GL_LINES which is presumably very fast but
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * won't render thickness correctly on some GPUs and has some quirks.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                */


var POINTS = 2,
    ATTRIBUTES = 3;

var EdgeFastProgram = function (_Program) {
  _inherits(EdgeFastProgram, _Program);

  function EdgeFastProgram(gl) {
    _classCallCheck(this, EdgeFastProgram);

    // Binding context
    var _this = _possibleConstructorReturn(this, (EdgeFastProgram.__proto__ || Object.getPrototypeOf(EdgeFastProgram)).call(this, gl, _edgeFastVert2.default, _edgeFastFrag2.default));

    _this.gl = gl;

    // Array data
    _this.array = null;

    // Initializing buffers
    _this.buffer = gl.createBuffer();
    gl.bindBuffer(gl.ARRAY_BUFFER, _this.buffer);

    // Locations
    _this.positionLocation = gl.getAttribLocation(_this.program, 'a_position');
    _this.colorLocation = gl.getAttribLocation(_this.program, 'a_color');
    _this.resolutionLocation = gl.getUniformLocation(_this.program, 'u_resolution');
    _this.matrixLocation = gl.getUniformLocation(_this.program, 'u_matrix');

    // Bindings
    gl.enableVertexAttribArray(_this.positionLocation);
    gl.enableVertexAttribArray(_this.colorLocation);

    gl.vertexAttribPointer(_this.positionLocation, 2, gl.FLOAT, false, ATTRIBUTES * Float32Array.BYTES_PER_ELEMENT, 0);
    gl.vertexAttribPointer(_this.colorLocation, 1, gl.FLOAT, false, ATTRIBUTES * Float32Array.BYTES_PER_ELEMENT, 8);
    return _this;
  }

  _createClass(EdgeFastProgram, [{
    key: 'allocate',
    value: function allocate(capacity) {
      this.array = new Float32Array(POINTS * ATTRIBUTES * capacity);
    }
  }, {
    key: 'process',
    value: function process(sourceData, targetData, data, offset) {

      var array = this.array;

      if (sourceData.hidden || targetData.hidden || data.hidden) {
        for (var l = i + POINTS * ATTRIBUTES; i < l; i++) {
          array[i] = 0;
        }
      }

      var x1 = sourceData.x,
          y1 = sourceData.y,
          x2 = targetData.x,
          y2 = targetData.y,
          color = (0, _utils.floatColor)(data.color);

      var i = POINTS * ATTRIBUTES * offset;

      // First point
      array[i++] = x1;
      array[i++] = y1;
      array[i++] = color;

      // Second point
      array[i++] = x2;
      array[i++] = y2;
      array[i] = color;
    }
  }, {
    key: 'bufferData',
    value: function bufferData() {
      var gl = this.gl;

      // Vertices data
      gl.bufferData(gl.ARRAY_BUFFER, this.array, gl.DYNAMIC_DRAW);
    }
  }, {
    key: 'render',
    value: function render(params) {
      var gl = this.gl;

      var program = this.program;
      gl.useProgram(program);

      // Binding uniforms
      gl.uniform2f(this.resolutionLocation, params.width, params.height);

      gl.uniformMatrix3fv(this.matrixLocation, false, params.matrix);

      // Drawing:
      gl.drawArrays(gl.LINES, 0, this.array.length / ATTRIBUTES);
    }
  }]);

  return EdgeFastProgram;
}(_program2.default);

exports.default = EdgeFastProgram;