'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _program = require('./program');

var _program2 = _interopRequireDefault(_program);

var _utils = require('../utils');

var _edgeTriangleVert = require('../shaders/edge.triangle.vert.glsl');

var _edgeTriangleVert2 = _interopRequireDefault(_edgeTriangleVert);

var _edgeTriangleFrag = require('../shaders/edge.triangle.frag.glsl');

var _edgeTriangleFrag2 = _interopRequireDefault(_edgeTriangleFrag);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; } /**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * Sigma.js WebGL Renderer Triangle Edge Program
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * ==============================================
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * Program rendering directed edges as a single anti-aliased triangle.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                */


var POINTS = 3,
    ATTRIBUTES = 9;

var EdgeTriangleProgram = function (_Program) {
  _inherits(EdgeTriangleProgram, _Program);

  function EdgeTriangleProgram(gl) {
    _classCallCheck(this, EdgeTriangleProgram);

    // Binding context
    var _this = _possibleConstructorReturn(this, (EdgeTriangleProgram.__proto__ || Object.getPrototypeOf(EdgeTriangleProgram)).call(this, gl, _edgeTriangleVert2.default, _edgeTriangleFrag2.default));

    _this.gl = gl;

    // Array data
    _this.array = null;

    // Initializing buffers
    _this.buffer = gl.createBuffer();
    gl.bindBuffer(gl.ARRAY_BUFFER, _this.buffer);

    // Locations
    _this.positionLocation = gl.getAttribLocation(_this.program, 'a_position');
    _this.normalLocation = gl.getAttribLocation(_this.program, 'a_normal');
    _this.thicknessLocation = gl.getAttribLocation(_this.program, 'a_thickness');
    _this.colorLocation = gl.getAttribLocation(_this.program, 'a_color');
    _this.barycentricLocation = gl.getAttribLocation(_this.program, 'a_barycentric');
    _this.resolutionLocation = gl.getUniformLocation(_this.program, 'u_resolution');
    _this.ratioLocation = gl.getUniformLocation(_this.program, 'u_ratio');
    _this.matrixLocation = gl.getUniformLocation(_this.program, 'u_matrix');
    _this.scaleLocation = gl.getUniformLocation(_this.program, 'u_scale');

    // Bindings
    gl.enableVertexAttribArray(_this.positionLocation);
    gl.enableVertexAttribArray(_this.normalLocation);
    gl.enableVertexAttribArray(_this.thicknessLocation);
    gl.enableVertexAttribArray(_this.colorLocation);
    gl.enableVertexAttribArray(_this.barycentricLocation);

    gl.vertexAttribPointer(_this.positionLocation, 2, gl.FLOAT, false, ATTRIBUTES * Float32Array.BYTES_PER_ELEMENT, 0);
    gl.vertexAttribPointer(_this.normalLocation, 2, gl.FLOAT, false, ATTRIBUTES * Float32Array.BYTES_PER_ELEMENT, 8);
    gl.vertexAttribPointer(_this.thicknessLocation, 1, gl.FLOAT, false, ATTRIBUTES * Float32Array.BYTES_PER_ELEMENT, 16);
    gl.vertexAttribPointer(_this.colorLocation, 1, gl.FLOAT, false, ATTRIBUTES * Float32Array.BYTES_PER_ELEMENT, 20);
    gl.vertexAttribPointer(_this.barycentricLocation, 3, gl.FLOAT, false, ATTRIBUTES * Float32Array.BYTES_PER_ELEMENT, 24);
    return _this;
  }

  _createClass(EdgeTriangleProgram, [{
    key: 'allocate',
    value: function allocate(capacity) {
      this.array = new Float32Array(POINTS * ATTRIBUTES * capacity);
    }
  }, {
    key: 'process',
    value: function process(sourceData, targetData, data, offset) {

      if (sourceData.hidden || targetData.hidden || data.hidden) {
        for (var l = i + POINTS * ATTRIBUTES; i < l; i++) {
          this.array[i] = 0;
        }
      }

      var thickness = data.size || 1,
          x1 = sourceData.x,
          y1 = sourceData.y,
          x2 = targetData.x,
          y2 = targetData.y,
          color = (0, _utils.floatColor)(data.color);

      // Computing normals
      var dx = x2 - x1,
          dy = y2 - y1;

      var len = dx * dx + dy * dy,
          n1 = 0,
          n2 = 0;

      if (len) {
        len = 1 / Math.sqrt(len);

        n1 = -dy * len;
        n2 = dx * len;
      }

      var i = POINTS * ATTRIBUTES * offset;

      var array = this.array;

      // TODO: I guess it's not necessary to pass normals to the shader here

      // First point
      array[i++] = x1;
      array[i++] = y1;
      array[i++] = n1;
      array[i++] = n2;
      array[i++] = thickness;
      array[i++] = color;
      array[i++] = 1;
      array[i++] = 0;
      array[i++] = 0;

      // Second point
      array[i++] = x1;
      array[i++] = y1;
      array[i++] = -n1;
      array[i++] = -n2;
      array[i++] = thickness;
      array[i++] = color;
      array[i++] = 0;
      array[i++] = 1;
      array[i++] = 0;

      // Third point
      array[i++] = x2;
      array[i++] = y2;
      array[i++] = 0;
      array[i++] = 0;
      array[i++] = 0;
      array[i++] = color;
      array[i++] = 0;
      array[i++] = 0;
      array[i] = 20;
    }
  }, {
    key: 'bufferData',
    value: function bufferData() {
      var gl = this.gl;

      // Vertices data
      gl.bufferData(gl.ARRAY_BUFFER, this.array, gl.DYNAMIC_DRAW);
    }
  }, {
    key: 'render',
    value: function render(params) {
      var gl = this.gl;

      var program = this.program;
      gl.useProgram(program);

      // Binding uniforms
      gl.uniform2f(this.resolutionLocation, params.width, params.height);
      gl.uniform1f(this.ratioLocation, params.ratio / Math.pow(params.ratio, params.edgesPowRatio));

      gl.uniformMatrix3fv(this.matrixLocation, false, params.matrix);

      gl.uniform1f(this.scaleLocation, params.ratio);

      // Drawing:
      gl.drawArrays(gl.TRIANGLES, 0, this.array.length / ATTRIBUTES);
    }
  }]);

  return EdgeTriangleProgram;
}(_program2.default);

exports.default = EdgeTriangleProgram;